﻿package 
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.display.StageQuality;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.events.SampleDataEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.net.navigateToURL;
	import flash.net.SharedObject;
	import flash.net.URLRequest;
	import flash.system.Capabilities;
	import flash.utils.ByteArray;
	import flash.utils.Endian;
	import flash.utils.getDefinitionByName;
	import flash.utils.getTimer;

	/**
	 * ...
	 * @author DefaultUser (Tools -> Custom Arguments...)
	 */
	public class DoomGame extends Sprite
	{
		[Embed(source="doomPreloader.swf", symbol="DoomLoader")]
		private var DoomPreloader:Class;
		[Embed(source = "doomPreloader.swf", symbol="LoadBar")]
		private var LoadBar:Class;
		[Embed(source = "doomPreloader.swf", symbol="Fader")]
		private var Fader:Class;
		[Embed(source = "doomPreloader.swf", symbol="LoadingText")]
		private var LoadingText:Class;
		[Embed(source = "doomPreloader.swf", symbol="PlayText")]
		private var PlayText:Class;
		[Embed(source = "doomPreloader.swf", symbol="UpdateFlashClip")]
		private var UpdateFlashClip:Class;
		
		private var _preloader:Sprite;
		private var _loadBar:Sprite;
		private var _loadingText:Sprite;
		private var _playText:Sprite;
		private var _fader:Sprite;
		private var _fadeTime:uint = 0;
		private var _waitTime:uint;
		
		private var _lib:Object;
		private var _bData:BitmapData;
		private var _memUser:Object;
		
		private var _soundBuffer:ByteArray;
		private var _soundBufferPos:uint = 0;
		private var _sound:Sound;
		private var _soundChannel:SoundChannel;
				
		private var _doomWad:ByteArray;
		
		private var _ram:ByteArray;
		
		private var _saveData:SharedObject;
		
		private var _keys:Array;
		
		private static const SCREENWIDTH:uint = 320;
		private static const SCREENHEIGHT:uint = 200;
		private static const NUMGAMESAMPLES:uint = 2048;
		private static const OVERLAPGAMESAMPLES:uint = 788;
		private static const FLASHSAMPLES:uint = 2048;
		private static const FLASHMINSAMPLES:uint = 2048;
		
		[SWF(width=960, height=600, backgroundColor=0, frameRate=35)]
	
		public function DoomGame()
		{
			if (uint(Capabilities.version.split(" ")[1].split(",")[0]) < 10)
			{
				var clip:Sprite = new UpdateFlashClip();
				clip.x = 470;
				clip.y = 300;
				
				addChild(clip);
				
				clip.addEventListener(MouseEvent.CLICK, updateButtonClicked);
				clip.mouseEnabled = true;
				clip.buttonMode = true;
				clip.useHandCursor = true;
			}
			else
				initPreloader();
		}
		
		public function updateButtonClicked(e:MouseEvent):void
		{
			flash.net.navigateToURL(new URLRequest("http://get.adobe.com/flashplayer/"), "_blank");
		}
		
		public function initPreloader():void
		{
			stage.quality = StageQuality.LOW;
			_preloader = new Sprite();
			_preloader.addChild(new DoomPreloader());
			_preloader.addChild(_loadBar = new LoadBar());
			//_preloader.addEventListener( MouseEvent.CLICK, goToNewgrounds );
			//_preloader.buttonMode = true;
			//_preloader.useHandCursor = true;
			_loadBar.x = 330;
			_loadBar.y = 435;
			_preloader.width = stage.stageWidth;
			_preloader.height = stage.stageHeight;
			_loadingText = new LoadingText();
			_loadingText.x = 330;
			_loadingText.y = 468;
			_preloader.addChild( _loadingText );
			var btn:Sprite = new LoadBar();
			btn.x = 325;
			btn.y = 106;
			btn.width = 306;
			btn.height = 306;
			btn.buttonMode = true;
			btn.alpha = 0;
			btn.addEventListener(MouseEvent.CLICK, goToNewgrounds);
			_preloader.addChild( btn );
			_playText = new PlayText();
			_playText.x = 400;
			_playText.y = 474;
			_playText.visible = false;
			_playText.buttonMode = true;
			_playText.addEventListener( MouseEvent.CLICK, playButtonClicked );
			_preloader.addChild(_playText);
			_loadBar.width = 3 * Math.floor( 100 * (loaderInfo.bytesLoaded / loaderInfo.bytesTotal) );
			addChild(_preloader);
			addEventListener(Event.ENTER_FRAME, tickPreloader);
		}
		
		private function tickPreloader(e:Event):void
		{
			_loadBar.width = 3 * Math.floor( 100 * (loaderInfo.bytesLoaded / loaderInfo.bytesTotal) );
			if ( loaderInfo.bytesLoaded == loaderInfo.bytesTotal )
			{
				_playText.visible = true;
				_loadingText.visible = false;

				if (_fader)
				{
					_fadeTime++;
					_fader.alpha = -1 + Math.floor( _fadeTime/35 / .2 )*.2;
					if (_fader.alpha >= 1.4)
					{
						_preloader.removeChild(_fader);
						_fader = null;
						_fadeTime = 0;
						removeChild(_preloader);
						removeEventListener(Event.ENTER_FRAME, tickPreloader);
						initGame();
					}
				}
			}
		}
		
		private function playButtonClicked(e:MouseEvent):void
		{
			if (!_fader)
			{
				_fader = new Fader();
				_fader.x = _fader.y = 0;
				_fader.width = stage.stageWidth;
				_fader.height = stage.stageHeight;
				_fader.alpha = 0;
				_preloader.addChild(_fader);
			}
		}
		
		private function initGame():void
		{
			var i:uint;
			
			
			_bData = new BitmapData(320, 200, false, 0);
			var bitmap:Bitmap = new Bitmap(_bData);
			addChild(bitmap);
			
			_soundBuffer = new ByteArray();
				
			_saveData = SharedObject.getLocal("Doom");
			if (!_saveData.data.saveGames)
			{
				_saveData.data.saveGames = new Array();
				for (i = 0; i < 6; i++)
					_saveData.data.saveGames[i] = new ByteArray();
			}
			
			var DoomDataClass:Class = Class(getDefinitionByName("DoomData"));
			var doomDataObject:* = new DoomDataClass();
			_lib = doomDataObject.lib;
			_memUser = doomDataObject.memUser;
			
			_keys = new Array();
			for (i = 0; i < 256; i++)
				_keys[i] = false;
					
			stage.addEventListener( KeyboardEvent.KEY_DOWN, keyDownHandler );
			stage.addEventListener( KeyboardEvent.KEY_UP, keyUpHandler );
			addEventListener(Event.ENTER_FRAME, enterFrameHandler);
			stage.frameRate = 35;
			
			var scale:Number = Math.min(stage.stageWidth/SCREENWIDTH, stage.stageHeight/SCREENHEIGHT);
			bitmap.scaleX = bitmap.scaleY = scale;
			
			_ram = _lib.getRam();
			
			_lib.setThiz( this );
			
			stage.focus = stage;
			
			_sound = new Sound();
			_sound.addEventListener( SampleDataEvent.SAMPLE_DATA, sampleDataHandler );
		}
		
		private function enterFrameHandler(e:Event):void
		{
			var ptr:uint;
			var i:uint, j:uint;
			
			_bData.lock();
			_lib.tick();
			
			var sampleShift:uint = OVERLAPGAMESAMPLES;
			
			if (!_soundChannel)
				sampleShift = 0;
			
			ptr = _lib.getSoundData() + (sampleShift * 4 * 2);
			_soundBuffer.writeBytes( _ram, ptr, (NUMGAMESAMPLES - sampleShift) * 2 * 4 );
			
			if (!_soundChannel)
			{
				_soundChannel = _sound.play();
				_soundChannel.addEventListener(Event.SOUND_COMPLETE, soundCompleteHandler);
			}
			
			ptr = _lib.getFrameBuffer();
			_ram.position = ptr;
			_bData.setPixels( _bData.rect, _ram );
			_bData.unlock();
		}
		
		private function sampleDataHandler(event:Event):void
		{
			var e:SampleDataEvent = SampleDataEvent(event);
			
			var l:uint = Math.min( FLASHSAMPLES * 2 * 4, _soundBuffer.length - _soundBufferPos  );
			if (l < FLASHMINSAMPLES * 2 * 4 )
			{return;
				for (var i:uint = 0; i < FLASHMINSAMPLES * 2; i++)
					e.data.writeFloat(0);
			}
			else
			{
				e.data.writeBytes(_soundBuffer, _soundBufferPos, l);
				_soundBufferPos += l;
				_soundBuffer.position = 0;
				_soundBuffer.writeBytes(_soundBuffer, _soundBufferPos);
				_soundBuffer.length = _soundBuffer.length - _soundBufferPos;
				_soundBufferPos = 0;
			}	
			
		}
		
		private function keyDownHandler(e:KeyboardEvent):void
		{
			if(_lib && !_keys[e.keyCode])
				_lib.keyDown( e.keyCode );
			_keys[e.keyCode] = true;
		}
		
		private function keyUpHandler(e:KeyboardEvent):void
		{
			if (_lib && _keys[e.keyCode])
				_lib.keyUp( e.keyCode );
				
			_keys[e.keyCode] = false;
		}
		
		public function getSaveGame(i:int, clear:Boolean):ByteArray
		{
			if (!_saveData.data.saveGames || !_saveData.data.saveGames[i])
				return new ByteArray();
			
			if (clear)_saveData.data.saveGames[i].clear();
			
			_saveData.data.saveGames[i].position = 0;
			return _saveData.data.saveGames[i];
		}
		
		public function goToNewgrounds(e:Event=null):void
		{
			navigateToURL(new URLRequest("http://www.newgrounds.com"), "_blank");
		}
		
		public function goToNewgroundsGames():void
		{
			navigateToURL(new URLRequest("http://www.newgrounds.com/game"), "_blank");
		}
		
		private function soundCompleteHandler(e:Event):void
		{
			// insist that our sound restarts next frame
			_soundChannel.removeEventListener(Event.SOUND_COMPLETE, soundCompleteHandler);
			_soundChannel = null;
		}
	}
	
}